/**
 * Library for the Mobile Processing project - http://mobile.processing.org
 *
 * Copyright (c) 2007 Shawn Van Every
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * @author  Shawn Van Every
 * 
 */

package com.mobvcasting.mvideocapture;

import java.io.ByteArrayOutputStream;

import processing.core.*;

import javax.microedition.media.*;
import javax.microedition.media.control.VideoControl;
import javax.microedition.media.control.RecordControl;

public class MVideoCapture 
{
	PMIDlet pMIDlet;
	Player mPlayer;
	VideoControl mVideoControl;
	RecordControl mRecordControl;
	
	ByteArrayOutputStream outStream;

	int videoWidth;
	int videoHeight;

	int canvasX;
	int canvasY;
	int canvasWidth;
	int canvasHeight;
	    
	CaptureTimer captureTimer;
	
	public static int CAPTURE_COMPLETE = 1;
	//public static int CAPTURE_IN_PROGRESS = 2;
	//public static int CAPTURE_NOT_STARTED = 3;
	
	/**
	 * Creates an MVideoCapture Object 
	 *
	 * @param  parent  a PMIDlet object, generally "this" in Mobile Processing
	 */	
	public MVideoCapture(PMIDlet parent) 
	{
		this.pMIDlet = parent;
		initialize("capture://video");
	}

	/**
	 * Creates an MVideoCapture Object 
	 *
	 * @param  parent  a PMIDlet object, generally "this" in Mobile Processing
	 * @param	locator a String containing a URL locator (capture://video)
	 * @see http://java.sun.com/javame/reference/apis/jsr135/javax/microedition/media/Manager.html
	 */	
	public MVideoCapture(PMIDlet parent, String locator) 
	{
		this.pMIDlet = parent;
		initialize(locator);
	}

	public String[] getControllers()
	{
		Control[] controls = mPlayer.getControls();
		String[] s_controls = new String[controls.length];
		
        for (int i = 0; i < controls.length; i++)
        {
        	s_controls[i] = controls[i].toString();
        }
        
        return s_controls;
	}
	
	static public String[] getLocators()
	{
		// http://www.forum.nokia.com/document/Java_ME_Developers_Library_v1/GUID-8F9BE454-FC5E-4B87-B805-765DEFA66C99/javax/microedition/media/Manager.html#media-protocol
		//Manager.getSupportedProtocols(content_type)
		//Manager.getSupportedContentTypes()
		//Manager.
		return Manager.getSupportedContentTypes("capture://video");
	}
	
	private void initialize(String locator)	
	{
		try
		{
            mPlayer = Manager.createPlayer(locator);
            mPlayer.realize();
			mVideoControl = (VideoControl)mPlayer.getControl("VideoControl");		

			mVideoControl.initDisplayMode(VideoControl.USE_DIRECT_VIDEO, pMIDlet.getCanvas());
            
            videoWidth = mVideoControl.getSourceWidth();
            videoHeight = mVideoControl.getSourceHeight();
                        
            canvasX = 0;
            canvasY = 0;
            canvasWidth = videoHeight;
            canvasHeight = videoWidth;

            mRecordControl = (RecordControl)mPlayer.getControl("RecordControl");	
		
		} 
		catch (Exception e) 
		{
			throw new RuntimeException(e.getMessage());
		}		
	}

	public void showCamera()
	{
		showCamera(canvasX,canvasY,canvasWidth,canvasHeight);
		
	}
	
	public void showCamera(int x, int y, int w, int h)
	{
		try 
		{
			mPlayer.start();
			mVideoControl.setVisible(true);			
			
			mVideoControl.setDisplayLocation(x, y);
			mVideoControl.setDisplaySize(w, h);
		} 
		catch (Exception e) 
		{ 
			throw new RuntimeException(e.getMessage() + e.toString());
		}		
	}
	
	public void startCapture()
	{
		if (mRecordControl != null)
		{
			outStream = new ByteArrayOutputStream();
			mRecordControl.setRecordStream(outStream);		
			
			if (mPlayer.getState() != Player.STARTED)
			{
				try
				{
					mPlayer.start();
				}
				catch (Exception e)
				{
					throw new RuntimeException(e.getMessage());
				}
			}
		
			mRecordControl.startRecord();
		}
	}

	public void hideCamera()
	{
		if (mPlayer.getState() == Player.STARTED)
		{
			try
			{
				mPlayer.stop();
			}
			catch (Exception e)
			{
				throw new RuntimeException(e.getMessage());
			}
		}
		mVideoControl.setVisible(false);
	}
	
	public void stopCapture()
	{
		if (mRecordControl != null)
		{
			mRecordControl.stopRecord();
			try 
			{
				mRecordControl.commit();
				mRecordControl.reset();
	
				// More cleanup?
				/*
				mRecordControl = null;
				System.gc();
				*/
				
				pMIDlet.libraryEvent(this, CAPTURE_COMPLETE, getCapturedVideo());
			} 
			catch (Exception e) 
			{
				throw new RuntimeException(e.getMessage());
			}
		}
	}
		
	public void closeCamera()
	{
		if (mPlayer.getState() != Player.CLOSED)
		{
			mPlayer.close();
		}
	}

	public byte[] getCapturedVideo()
	{
		return outStream.toByteArray(); 
	}
	
	public void timedCapture(int seconds)
	{
        captureTimer = new CaptureTimer(seconds);
        startCapture();
        captureTimer.start();		
	}
	
	public class CaptureTimer extends Thread
    {
        int increment;
        int times;
        
        public CaptureTimer(int _seconds)
        {
        	this(1000,_seconds);
        }
        
        public CaptureTimer(int _increment, int _times)
        {
            increment = _increment;
            times = _times;
        }
		
        public void run()
        {
            for(int i = 0; i < times; i++)
            {
                try
                {
                    Thread.sleep(increment * 1000);
                }
                catch(InterruptedException interruptedexception) { }
            }
            timerComplete();
        }

        public void timerComplete()
        {
            stopCapture();
        }
    }
}